#include-once
;===========================================================================
; AutoIt implenentation by ProgAndy
; Author: ProgAndy
;===========================================================================
#cs
#ifndef __CURL_EASY_H
#define __CURL_EASY_H
/***************************************************************************
 *                                  _   _ ____  _
 *  Project                     ___| | | |  _ \| |
 *                             / __| | | | |_) | |
 *                            | (__| |_| |  _ <| |___
 *                             \___|\___/|_| \_\_____|
 *
 * Copyright (C) 1998 - 2008, Daniel Stenberg, <daniel@haxx.se>, et al.
 *
 * This software is licensed as described in the file COPYING, which
 * you should have received as part of this distribution. The terms
 * are also available at http://curl.haxx.se/docs/copyright.html.
 *
 * You may opt to use, copy, modify, merge, publish, distribute and/or sell
 * copies of the Software, and permit persons to whom the Software is
 * furnished to do so, under the terms of the COPYING file.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 * $Id: easy.h,v 1.14 2008-05-12 21:43:28 bagder Exp $
 ***************************************************************************/
#ifdef  __cplusplus
extern "C" {
#endif
#ce

;~ CURL_EXTERN CURL *curl_easy_init(void);
Func _curl_easy_init()
	Local $aResult = DllCall($ghLIBCURLdll, "ptr:cdecl", "curl_easy_init")
	If @error Then Return SetError(1,0,0)
	Return $aResult[0]
EndFunc

;~ CURL_EXTERN CURLcode curl_easy_setopt(CURL *curl, CURLoption option, ...);
Func _curl_easy_setopt($curl, $option, $vValue)
	$tpValue = 'ptr'
	If IsString($vValue) Then $tpValue = 'str'
	Local $aResult = DllCall($ghLIBCURLdll, "int:cdecl", "curl_easy_setopt", "ptr", $curl, "int", $option, $tpValue, $vValue)
	If @error Then Return SetError(1,0,$CURL_LAST)
	Return $aResult[0]
EndFunc

;~ CURL_EXTERN CURLcode curl_easy_perform(CURL *curl);
Func _curl_easy_perform($curl)
	Local $aResult = DllCall($ghLIBCURLdll, "int:cdecl", "curl_easy_perform", "ptr", $curl)
	If @error Then Return SetError(1,0,$CURL_LAST)
	Return $aResult[0]
EndFunc
;~ CURL_EXTERN void curl_easy_cleanup(CURL *curl);
Func _curl_easy_cleanup($curl)
	Local $aResult = DllCall($ghLIBCURLdll, "none:cdecl", "curl_easy_cleanup", "ptr", $curl)
	If @error Then Return SetError(1)
EndFunc

#cs /*
 * NAME curl_easy_getinfo()
 *
 * DESCRIPTION
 *
 * Request internal information from the curl session with this function.  The
 * third argument MUST be a pointer to a long, a pointer to a char * or a
 * pointer to a double (as the documentation describes elsewhere).  The data
 * pointed to will be filled in accordingly and can be relied upon only if the
 * function returns CURLE_OK.  This function is intended to get used *AFTER* a
 * performed transfer, all results from this function are undefined until the
 * transfer is completed.
#ce */
;~ CURL_EXTERN CURLcode curl_easy_getinfo(CURL *curl, CURLINFO info, ...);
Func _curl_easy_getinfo($curl, $info, ByRef $vRecvVal)
	Local $tpValue = 'ptr*'
	Select
		Case $info >= $CURLINFO_SLIST
			$tpValue = 'ptr*'
		Case $info >= $CURLINFO_DOUBLE
			$tpValue = 'double*'
		Case $info >= $CURLINFO_LONG
			$tpValue = 'long*'
		Case $info >= $CURLINFO_STRING
			$tpValue = 'str*'
	EndSelect

	Local $aResult = DllCall($ghLIBCURLdll, "int:cdecl", "curl_easy_getinfo", "ptr", $curl, "int", $info, $tpValue, $vRecvVal)
	If @error Then Return SetError(1,0,$CURL_LAST)
	$vRecvVal = $aResult[3]
	Return $aResult[0]
EndFunc

#cs /*
 * NAME curl_easy_duphandle()
 *
 * DESCRIPTION
 *
 * Creates a new curl session handle with the same options set for the handle
 * passed in. Duplicating a handle could only be a matter of cloning data and
 * options, internal state info and things like persistant connections cannot
 * be transfered. It is useful in multithreaded applications when you can run
 * curl_easy_duphandle() for each new thread to avoid a series of identical
 * curl_easy_setopt() invokes in every thread.
#ce */
;~ CURL_EXTERN CURL* curl_easy_duphandle(CURL *curl);
Func _curl_easy_duphandle($curl)
	Local $aResult = DllCall($ghLIBCURLdll, "ptr:cdecl", "curl_easy_duphandle", "ptr", $curl)
	If @error Then Return SetError(1,0,0)
	Return $aResult[0]
EndFunc

#cs /*
 * NAME curl_easy_reset()
 *
 * DESCRIPTION
 *
 * Re-initializes a CURL handle to the default values. This puts back the
 * handle to the same state as it was in when it was just created.
 *
 * It does keep: live connections, the Session ID cache, the DNS cache and the
 * cookies.
#ce */
;~ CURL_EXTERN void curl_easy_reset(CURL *curl);
Func _curl_easy_reset($curl)
	Local $aResult = DllCall($ghLIBCURLdll, "none:cdecl", "curl_easy_reset", "ptr", $curl)
	If @error Then Return SetError(1)
EndFunc


#cs /*
 * NAME curl_easy_recv()
 *
 * DESCRIPTION
 *
 * Receives data from the connected socket. Use after successful
 * curl_easy_perform() with CURLOPT_CONNECT_ONLY option.
#ce */
;~ CURL_EXTERN CURLcode curl_easy_recv(CURL *curl, void *buffer, size_t buflen,
;~                                     size_t *n);
Func _curl_easy_recv($curl, $pBuffer, $buflen, ByRef $n)
	Local $aResult = DllCall($ghLIBCURLdll, "ptr:cdecl", "curl_easy_recv", "ptr", $curl, "ptr", $pBuffer, "uint", $buflen, "uint*", $n)
	If @error Then Return SetError(1,0,$CURL_LAST)
	$n = $aResult[4]
	Return $aResult[0]
EndFunc

#cs /*
 * NAME curl_easy_send()
 *
 * DESCRIPTION
 *
 * Sends data over the connected socket. Use after successful
 * curl_easy_perform() with CURLOPT_CONNECT_ONLY option.
#ce */
;~ CURL_EXTERN CURLcode curl_easy_send(CURL *curl, const void *buffer,
;~                                     size_t buflen, size_t *n);
Func _curl_easy_send($curl, $pBuffer, $buflen, ByRef $n)
	Local $aResult = DllCall($ghLIBCURLdll, "ptr:cdecl", "curl_easy_send", "ptr", $curl, "ptr", $pBuffer, "uint", $buflen, "uint*", $n)
	If @error Then Return SetError(1,0,$CURL_LAST)
	$n = $aResult[4]
	Return $aResult[0]
EndFunc

#cs
#ifdef  __cplusplus
}
#endif

#endif
#ce
